#!/usr/bin/env python3

import argparse
import subprocess
import os
import glob
import json


from paignion.definitions import (
    FRONTEND_DIR,
    SIMPLE_ORIGIN_ROOM_TEMPLATE,
    SIMPLE_SECOND_ROOM_TEMPLATE,
    __version__,
)
from paignion.parser import PaignionParser


def paignion_init(namespace):
    project_dir = namespace.project_dir

    print(f"Initializing new Paignion game at directory {project_dir}")
    subprocess.run(["mkdir", "-p", project_dir], stdout=subprocess.PIPE)
    subprocess.run(
        ["mkdir", "-p", os.path.join(project_dir, "rooms")], stdout=subprocess.PIPE
    )

    with open(os.path.join(project_dir, "rooms", "origin.md"), "w") as map_file:
        map_file.write(SIMPLE_ORIGIN_ROOM_TEMPLATE.format(__version__))

    with open(os.path.join(project_dir, "rooms", "second_room.md"), "w") as map_file:
        map_file.write(SIMPLE_SECOND_ROOM_TEMPLATE.format(__version__))


def paignion_build(namespace):
    parser = PaignionParser()

    parser.verify_project_dir(namespace.project_dir)

    room_files = [
        f for f in glob.glob(os.path.join(namespace.project_dir, "rooms", "*.md"))
    ]

    # Generate final GAME_DATA object
    GAME_DATA = parser.parse_room_files(room_files)

    build_dir = os.path.join(namespace.project_dir, "build")
    print(f"Building project {namespace.project_dir}")
    subprocess.run(["mkdir", "-p", build_dir], stdout=subprocess.PIPE)
    subprocess.run(
        [
            "cp",
            os.path.join(FRONTEND_DIR, "index.html"),
            os.path.join(FRONTEND_DIR, "main.css"),
            os.path.join(FRONTEND_DIR, "paignion.js"),
            build_dir,
        ],
        stdout=subprocess.PIPE,
    )

    # Put game data into the generated paignion.js file
    with open(os.path.join(build_dir, "paignion.js"), "a") as f:
        f.write("\n\n\n// Automatically generated game data object\n")
        f.write(f"let GAME_DATA = {json.dumps(GAME_DATA)};")

    print("Done")


def paignion_main_function():
    parser = argparse.ArgumentParser(description="A text adventure game engine")
    subparsers = parser.add_subparsers(
        help="Run `paignion init <game_dir>` to start a new game project, or "
        "`paignion build <game_dir>` to build game files into a playable game"
    )

    parser.add_argument(
        "-v",
        "--version",
        action="version",
        version=__version__,
        help="Show the version of Paignion and exit",
    )

    parser_init = subparsers.add_parser("init", help="Create a new game project")
    parser_init.set_defaults(func=paignion_init)
    parser_init.add_argument(
        "project_dir", help="The project directory (to be created if it doesn't exist)"
    )

    parser_build = subparsers.add_parser(
        "build", help="Build an existing game project into a playable game"
    )
    parser_build.set_defaults(func=paignion_build)
    parser_build.add_argument(
        "project_dir", help="The directory containing the project files"
    )

    args = parser.parse_args()
    if "func" not in args:
        # No valid sub-command was provided, show help
        parser.print_help()
        exit(0)

    args.func(args)


if __name__ == "__main__":
    paignion_main_function()



from paignion.exception import PaignionException


class PaignionItem(object):
    def __init__(
        self, name, description, amount=1, visible=True, effect=None, used_with=None
    ):
        self.name = name
        self.description = description
        self.amount = amount
        self.visible = visible
        self.effect = effect
        self.used_with = used_with

        self.verify_attributes()

    def verify_attributes(self):
        # Amount should be 1 by default
        self.amount = 1 if not self.amount else self.amount
        # Item should be visible by default
        self.visible = True if self.visible == None else self.visible
        # Used with should be an empty list by default
        self.used_with = [] if not self.used_with else self.used_with

        # Item name is mandatory
        if not self.name:
            raise PaignionException("Name missing for item")

        # Item description is mandatory
        if self.visible and not self.description:
            raise PaignionException(f"Description missing for item `{self.name}`")

    def dump(self):
        return {
            "name": self.name,
            "description": self.description,
            "amount": self.amount,
            "visible": self.visible,
            "effect": self.effect,
            "used_with": [i.dump() for i in self.used_with],
        }

    def __str__(self):
        return json.dumps(self.dump(), indent=4)



import re
import os
import json
import yaml
import markdown

from paignion.definitions import DIRECTIONS, MD_EXTENSIONS
from paignion.exception import PaignionException
from paignion.room import PaignionRoom
from paignion.item import PaignionItem
from paignion.used_with_item import PaignionUsedWithItem


class PaignionParser(object):
    def verify_project_dir(self, project_dir):
        rooms_dir = os.path.join(project_dir, "rooms")

        # Check that the rooms directory exists
        if not os.path.exists(rooms_dir) or not os.path.isdir(rooms_dir):
            raise PaignionException(
                "Rooms directory not found. Please create the rooms/ directory."
            )

    def parse_room_files(self, room_files):
        room_data = {}

        for room_file in room_files:
            with open(room_file, "r") as f:
                room_data.update(
                    self.parse_room_file(
                        room_data=f.read(),
                        room_name=os.path.splitext(room_file)[0].split("/")[-1],
                    )
                )

        return room_data

    def markdownify(self, string):
        if not string:
            return string

        return markdown.markdown(string, extensions=MD_EXTENSIONS)

    def parse_room_file(self, room_data, room_name):
        # Detect YAML part
        frontmatter_match = re.search("---\n(((?!---)[\\s\\S])*)---", room_data)

        # Parse YAML part
        frontmatter = yaml.load(frontmatter_match.group(1), Loader=yaml.Loader)
        # If the YAML is empty, replace it by an empty dict
        if not frontmatter:
            frontmatter = {}

        # Parse Markdown part
        md = self.markdownify(room_data[frontmatter_match.span()[1] :])

        # Parse items
        if "items" in frontmatter:
            # Parse tangible items
            tangible_items = []
            raw_tangible_items = (
                frontmatter["items"]["tangible"]
                if "tangible" in frontmatter["items"]
                else []
            )
            for tangible_item in raw_tangible_items:
                # Parse used_with items
                if "used_with" in tangible_item:
                    used_with = []
                    for uw_item in tangible_item["used_with"]:
                        try:
                            used_with.append(
                                PaignionUsedWithItem(
                                    name=uw_item.get("name"),
                                    effect_message=self.markdownify(
                                        uw_item.get("effect_message")
                                    ),
                                    consumes_subject=uw_item.get("consumes_subject"),
                                    consumes_object=uw_item.get("consumes_object"),
                                    actions=uw_item.get("actions"),
                                )
                            )
                        except PaignionException:
                            raise PaignionException(
                                f"Error parsing used_with item for tangible item"
                                f"`{tangible_item.get('name')}` in room `{room_name}`"
                            )
                else:
                    used_with = []

                try:
                    tangible_items.append(
                        PaignionItem(
                            name=tangible_item.get("name"),
                            description=self.markdownify(
                                tangible_item.get("description")
                            ),
                            amount=tangible_item.get("amount"),
                            visible=tangible_item.get("visible"),
                            effect=tangible_item.get("effect"),
                            used_with=used_with,
                        )
                    )
                except PaignionException:
                    raise PaignionException(
                        f"Error parsing tangible item `{tangible_item.get('name')}` in"
                        f"room `{room_name}`"
                    )

            # Parse intangible items
            intangible_items = []
            raw_intangible_items = (
                frontmatter["items"]["intangible"]
                if "intangible" in frontmatter["items"]
                else []
            )
            for intangible_item in raw_intangible_items:
                # Parse used_with items
                if "used_with" in intangible_item:
                    used_with = []
                    for uw_item in intangible_item["used_with"]:
                        try:
                            used_with.append(
                                PaignionUsedWithItem(
                                    name=uw_item.get("name"),
                                    effect_message=self.markdownify(
                                        uw_item.get("effect_message")
                                    ),
                                    consumes_subject=uw_item.get("consumes_subject"),
                                    consumes_object=uw_item.get("consumes_object"),
                                    actions=uw_item.get("actions"),
                                )
                            )
                        except PaignionException:
                            raise PaignionException(
                                f"Error parsing used_with item for intangible item"
                                f"`{intangible_item}` in room `{room_name}`"
                            )
                else:
                    used_with = []

                try:
                    intangible_items.append(
                        PaignionItem(
                            name=intangible_item.get("name"),
                            description=self.markdownify(
                                intangible_item.get("description")
                            ),
                            amount=intangible_item.get("amount"),
                            visible=intangible_item.get("visible"),
                            effect=intangible_item.get("effect"),
                            used_with=used_with,
                        )
                    )
                except PaignionException:
                    raise PaignionException(
                        f"Error parsing intangible item `{intangible_item.get('name')}`"
                        f" in room `{room_name}`"
                    )
        else:
            tangible_items = []
            intangible_items = []

        # Create room
        try:
            room = PaignionRoom(
                name=room_name,
                description=md,
                north=frontmatter.get("north"),
                east=frontmatter.get("east"),
                south=frontmatter.get("south"),
                west=frontmatter.get("west"),
                up=frontmatter.get("up"),
                down=frontmatter.get("down"),
                tangible_items=tangible_items,
                intangible_items=intangible_items,
            )
        except PaignionException:
            raise PaignionException(f"Error parsing room `{room_name}`")

        return room.dump()
